(function() {
    printLog("inner_bridge begin");
	
    var messagingIframe;
    var messagingIframeMessage;
	
	var CUSTOM_PROTOCOL_SCHEME = 'qihoo_js_bridge';
	var CUSTOM_ID_HAS_MESSAGE = '__has_message__/';
	var CUSTOM_ID_REUTRN = 'return/';
	var CUSTOM_ID_FETCH_QUEUE = 'fetchQueue/';
	
	var sendMessageQueue = [];
	var receiveMessageQueue = [];
	var callbacks = {};
	var handlers = {};
	var seq = 1;

    function _innerBridgeReadyIframe(doc) {
        messagingIframe = doc.createElement('iframe');
        messagingIframe.style.display = 'none';
        doc.documentElement.appendChild(messagingIframe);
		
        messagingIframeMessage = doc.createElement('iframe');
        messagingIframeMessage.style.display = 'none';
        doc.documentElement.appendChild(messagingIframeMessage);
    }
	
	function _doSend(message, responseCallback) {
	    if (responseCallback) {
            var callbackId = 'cb_' + (seq++) + '_' + new Date().getTime();
			printLog("inner_bridge _doSend callbackId:" + callbackId);
            callbacks[callbackId] = responseCallback;
            message.callbackId = callbackId;
        }

		sendMessageQueue.push(message);
		messagingIframe.src = CUSTOM_PROTOCOL_SCHEME + '://' + CUSTOM_ID_HAS_MESSAGE;
    }
	
	function _dispatchMessageFromNative(messageJSON) {
        printLog("inner_bridge dispatchMessageFromNative messageJSON:" + messageJSON);
		setTimeout(function() {
			var message = JSON.parse(messageJSON);
			if (message.responseId) {
                callback = callbacks[message.responseId];
                if (callback != null) {
					callback(message.responseData);
					delete callbacks[message.responseId];
                    return;
                }
            } else {
			    // 直接发送
				var responseCallback = function(responseData) {
							if (message.callbackId != null) {
								_doSend({
									responseId: message.callbackId,
									responseData: responseData
								});
							} else {
								// do nothing
							}
						};

			    var handler;
				if (message.handlerName != null) {
                    handler = handlers[message.handlerName];
                } else {
				    handler = InnerBridge._messageHandler;
				}
				if (handler != null) {
                    handler(message.requestData, responseCallback);
                }
			}
		});
    }
	
	function _init(defaultMessageHandler) {
	    printLog("inner_bridge _init");
	    if (InnerBridge._messageHandler != null) {
            throw new Error('InnerBridge.init called twice');
        }
        InnerBridge._messageHandler = defaultMessageHandler;
		var receivedMessages = receiveMessageQueue;
        receiveMessageQueue = null;
        for (var i = 0; i < receivedMessages.length; i++) {
            _dispatchMessageFromNative(receivedMessages[i]);
        }
    }
	
    function _registerHandler(handlerName, handler) {
		printLog("inner_bridge _registerHandler handlerName:" + handlerName);
		handlers[handlerName] = handler;
    }
	
    function _callHandler(handlerName, requestData, responseCallback) {
        printLog("inner_bridge _callHandler handlerName:" + handlerName);
		printLog("inner_bridge _callHandler requestData:" + requestData);
        _doSend({
            handlerName: handlerName,
            requestData: requestData
        }, responseCallback);
    }
	
    function _fetchQueue() {
        printLog("inner_bridge _fetchQueue");
		var messageQueueString = JSON.stringify(sendMessageQueue);
        sendMessageQueue = [];
        
		messagingIframeMessage.src = CUSTOM_PROTOCOL_SCHEME + '://' + CUSTOM_ID_REUTRN + CUSTOM_ID_FETCH_QUEUE + encodeURIComponent(messageQueueString);
    }
	
	function _handleMessageFromNative(messageJSON) {
	    printLog("inner_bridge handleMessageFromNative");
		if (receiveMessageQueue) {
            receiveMessageQueue.push(messageJSON);
        } else {
            _dispatchMessageFromNative(messageJSON);
        }
	}
	
    var InnerBridge = window.InnerBridge = {
	    init: _init,
        registerHandler: _registerHandler,
        callHandler: _callHandler,
		fetchQueue: _fetchQueue,
        handleMessageFromNative: _handleMessageFromNative
    };

    var doc = document;
    _innerBridgeReadyIframe(doc);
	
	var readyEvent = doc.createEvent('Events');
    readyEvent.initEvent('InnerBridgeReady');
    doc.dispatchEvent(readyEvent);
	
	printLog("inner_bridge end");
})();